@@ -54,7 +54,7 @@ int clean_client_dir( void );
 int check( SNET *sn, char *type, char *path); 
 int createspecial( SNET *sn, struct list *special_list );
 int getstat( SNET *sn, char *description, char *stats );
-int read_kfile( char * );
+int read_kfile( char *, char * );
 SNET *sn;
 
 void			(*logger)( char * ) = NULL;
@@ -189,8 +189,11 @@ cleandirs( char *path, struct llist *khead )
 	}
 
 	for ( kcur = khead; kcur != NULL; kcur = kcur->ll_next ) {
-	    if ( strcmp( cur->ll_name, kcur->ll_name ) == 0
-			|| ischild( kcur->ll_name, cur->ll_name )) {
+	    if (( case_sensitive &&
+			strcmp( cur->ll_name, kcur->ll_name ) == 0 ) ||
+		    ( !case_sensitive &&
+			strcasecmp( cur->ll_name, kcur->ll_name ) == 0 ) ||
+		    ischildcase( kcur->ll_name, cur->ll_name, case_sensitive)) {
 		match = 1;
 		break;
 	    }
@@ -234,6 +237,7 @@ clean_client_dir( void )
 
     while (( node = list_pop_head( kfile_seen )) != NULL ) {
 	expand_kfile( &khead, node->n_path );
+	free( node );
     }
 
     /*
@@ -321,7 +325,7 @@ createspecial( SNET *sn, struct list *special_list )
 	    return( 1 );
 	}
 
-	if ( getstat( sn, (char *)&filedesc, stats ) != 0 ) {
+	if ( getstat( sn, filedesc, stats ) != 0 ) {
 	    return( 1 );
 	}
 
@@ -447,7 +451,7 @@ check( SNET *sn, char *type, char *file )
 	strcpy( path, base_kfile );
     }
 
-    if ( getstat( sn, (char *)&pathdesc, stats ) != 0 ) {
+    if ( getstat( sn, pathdesc, stats ) != 0 ) {
 	return( 2 );
     }
     tac = acav_parse( NULL, stats, &targv );
@@ -466,7 +470,7 @@ check( SNET *sn, char *type, char *file )
 	    /* Local file is missing */
 	    if ( update ) {
 		if ( !quiet ) { printf( "%s:", path ); fflush( stdout ); }
-		if ( retr( sn, pathdesc, path, (char *)&tempfile, 0666, 
+		if ( retr( sn, pathdesc, path, tempfile, 0666, 
 			strtoofft( targv[ 6 ], NULL, 10 ), targv[ 7 ] ) != 0 ) {
 		    return( 2 );
 		}
@@ -514,7 +518,7 @@ check( SNET *sn, char *type, char *file )
 		perror( path );
 		return( 2 );
 	    }
-	    if ( retr( sn, pathdesc, path, (char *)&tempfile, 0666, 
+	    if ( retr( sn, pathdesc, path, tempfile, 0666, 
 		    strtoofft( targv[ 6 ], NULL, 10 ), targv[ 7 ] ) != 0 ) {
 		return( 2 );
 	    }
@@ -559,9 +563,10 @@ main( int argc, char **argv )
     char		path[ MAXPATHLEN ];
     char		tempfile[ MAXPATHLEN ];
     char	        **capa = NULL;		/* capabilities */
+    char		*event = "ktcheck";	/* report event type */
 
     while (( c = getopt( argc, argv,
-	    "Cc:D:h:iK:np:P:qrvVw:x:y:z:Z:" )) != EOF ) {
+	    "Cc:D:e:h:IiK:np:P:qrvVw:x:y:z:Z:" )) != EOF ) {
 	switch( c ) {
 	case 'C':	/* clean up dir containing command.K */
 	    clean = 1;
@@ -581,10 +586,18 @@ main( int argc, char **argv )
 	    radmind_path = optarg;
 	    break;
 
+	case 'e':		/* set the event label for reporting */
+	    event = optarg;
+	    break;
+
 	case 'h':
 	    host = optarg;
 	    break;
 
+	case 'I':
+	    case_sensitive = 0;
+	    break;
+
 	case 'i':
 	    setvbuf( stdout, ( char * )NULL, _IOLBF, 0 );
 	    break;
@@ -673,7 +686,7 @@ main( int argc, char **argv )
 
     if ( err || ( argc - optind != 0 )) {
 	fprintf( stderr, "usage: %s ", argv[ 0 ] );
-	fprintf( stderr, "[ -CinrV ] [ -q | -v ] " );
+	fprintf( stderr, "[ -CIinrV ] [ -q | -v ] " );
 	fprintf( stderr, "[ -c checksum ] [ -D radmind_path ] " );
 	fprintf( stderr, "[ -K command file ] " );
 	fprintf( stderr, "[ -h host ] [ -p port ] [ -P ca-pem-directory ] " );
@@ -755,11 +768,15 @@ main( int argc, char **argv )
 	break;
 
     case 2:
-	if ( report ) report_event( sn, "ktcheck", "Error" );
+	if ( report ) {
+	    if ( report_event( sn, event, "Error" ) != 0 ) {
+		fprintf( stderr, "warning: could not report event\n" );
+	    }
+	}
 	exit( 2 );
     }
 
-    if ( read_kfile( base_kfile ) != 0 ) {
+    if ( read_kfile( base_kfile, event ) != 0 ) {
 	exit( 2 );
     }
 
@@ -797,14 +814,15 @@ main( int argc, char **argv )
 			exit( 2 );
 		    }
 		    if ( !quiet ) printf( "%s: created\n", path ); 
-		    change++;
 		} else {
 		    /* special.T not updated */
 		    if ( unlink( tempfile ) !=0 ) {
 			perror( tempfile );
 			exit( 2 );
 		    }
+		    if ( !quiet ) printf( "%s: missing\n", path );
 		}
+		change++;
 		goto done;
 	    }
 	    perror( path );
@@ -845,13 +863,14 @@ main( int argc, char **argv )
 		}
 		if ( !quiet ) printf( "%s: updated\n", path ); 
 	    } else {
+		if ( !quiet ) printf( "%s: out of date\n", path );
 		if ( unlink( tempfile ) !=0 ) {
 		    perror( tempfile );
 		    exit( 2 );
 		}
 	    }
 	} else {
-	    /* special.T not updated */
+	    /* local special.T correct */
 	    if ( unlink( tempfile ) !=0 ) {
 		perror( tempfile );
 		exit( 2 );
@@ -871,20 +890,20 @@ done:
     if ( change ) {
 	if ( update ) {
 	    if ( report ) {
-		if ( report_event( sn, "ktcheck", "Updates retrieved" ) != 0 ) {
+		if ( report_event( sn, event, "Updates retrieved" ) != 0 ) {
 		    fprintf( stderr, "warning: could not report event\n" );
 		}
 	    }
 	} else {
 	    if ( report ) {
-		if ( report_event( sn, "ktcheck", "Updates available" ) != 0 ) {
+		if ( report_event( sn, event, "Updates available" ) != 0 ) {
 		    fprintf( stderr, "warning: could not report event\n" );
 		}
 	    }
 	}
     } else {
 	if ( report ) {
-	    if ( report_event( sn, "ktcheck", "No updates needed" ) != 0 ) {
+	    if ( report_event( sn, event, "No updates needed" ) != 0 ) {
 		fprintf( stderr, "warning: could not report event\n" );
 	    }
 	}
@@ -904,7 +923,7 @@ done:
 }
 
     int
-read_kfile( char * kfile )
+read_kfile( char * kfile,  char * event )
 {
     int		ac, minus = 0;
     char	**av;
@@ -976,10 +995,14 @@ read_kfile( char * kfile )
 		}
 		break;
 	    case 2:
-		if ( report ) report_event( sn, "ktcheck", "Error" );
+		if ( report ) {
+		    if ( report_event( sn, event, "Error" ) != 0 ) {
+			fprintf( stderr, "warning: could not report event\n" );
+		    }
+		}
 		goto error;
 	    }
-	    if ( read_kfile( path ) != 0 ) {
+	    if ( read_kfile( path, event ) != 0 ) {
 		exit( 2 );
 	    }
 	    break;
@@ -992,7 +1015,8 @@ read_kfile( char * kfile )
 		}
 	    } else {
 		if ( !list_check( special_list, av[ 1 ] )) {
-		    if ( list_insert( special_list, av[ 1 ] ) != 0 ) {
+		    if ( list_insert_case( special_list, av[ 1 ],
+				case_sensitive ) != 0 ) {
 			perror( "list_insert" );
 			exit( 2 );
 		    }
@@ -1012,7 +1036,11 @@ read_kfile( char * kfile )
 		}
 		break;
 	    case 2:
-		if ( report ) report_event( sn, "ktcheck", "Error" );
+		if ( report ) {
+		    if ( report_event( sn, event, "Error" ) != 0 ) {
+			fprintf( stderr, "warning: could not report event\n" );
+		    }
+		}
 		exit( 2 );
 	    }
 	    break;
@@ -1037,7 +1065,11 @@ done:
 	return( -1 );
     }
     if ( !update && change ) {
-	if ( report ) report_event( sn, "ktcheck", "Updates available" );
+	if ( report ) {
+	    if ( report_event( sn, event, "Updates available" ) != 0 ) {
+		fprintf( stderr, "warning: could not report event\n" );
+	    }
+	}
 	exit( 1 );
     }
     return( 0 );
