@@ -142,13 +142,13 @@ do_line( char *tline, int present, struct stat *st, SNET *sn )
 
 	if ( special ) {
 	    if ( snprintf( pathdesc, MAXPATHLEN * 2, "SPECIAL %s",
-		    targv[ 1 ]) > ( MAXPATHLEN * 2 ) - 1 ) {
+		    targv[ 1 ]) >= ( MAXPATHLEN * 2 )) {
 		fprintf( stderr, "SPECIAL %s: too long\n", targv[ 1 ]);
 		return( 1 );
 	    }
 	} else {
 	    if ( snprintf( pathdesc, MAXPATHLEN * 2, "FILE %s %s",
-		    transcript, targv[ 1 ]) > ( MAXPATHLEN * 2 ) -1 ) {
+		    transcript, targv[ 1 ]) >= ( MAXPATHLEN * 2 )) {
 		fprintf( stderr, "FILE %s %s: command too long\n",
 		    transcript, targv[ 1 ]);
 		return( 1 );
@@ -167,7 +167,7 @@ do_line( char *tline, int present, struct stat *st, SNET *sn )
 		break;
 	    }
 	} else {
-	    switch ( retr( sn, pathdesc, path, (char *)&temppath, 0600,
+	    switch ( retr( sn, pathdesc, path, temppath, 0600,
 		strtoofft( targv[ 6 ], NULL, 10 ), cksum_b64 )) {
 	    case -1:
 		/* Network problem */
@@ -250,10 +250,11 @@ main( int argc, char **argv )
     int			authlevel = _RADMIND_AUTHLEVEL;
     int			force = 0;
     int			use_randfile = 0;
-	char        **capa = NULL; /* capabilities */
+    char	        **capa = NULL;		/* capabilities */
+    char		* event = "lapply";	/* report event type */
 
     while (( c = getopt( argc, argv,
-	    "%c:CFh:iInp:P:qru:Vvw:x:y:z:Z:" )) != EOF ) {
+	    "%c:Ce:Fh:iInp:P:qru:Vvw:x:y:z:Z:" )) != EOF ) {
 	switch( c ) {
 	case '%':
 	    showprogress = 1;
@@ -273,6 +274,10 @@ main( int argc, char **argv )
 	    create_prefix = 1;
 	    break;
 
+	case 'e':		/* set the event label for reporting */
+	    event = optarg;
+	    break;
+
 	case 'F':
 	    force = 1;
 	    break;
@@ -728,7 +733,7 @@ filechecklist:
 
     if ( network ) {
 	if ( report ) {
-	    if ( report_event( sn, "lapply",
+	    if ( report_event( sn, event,
 		    "Changes applied successfully" ) != 0 ) {
 		fprintf( stderr, "warning: could not report event\n" );
 	    }
@@ -753,14 +758,13 @@ error1:
 #endif /* HAVE_ZLIB */
 	if ( change ) {
 	    if ( network && report ) {
-		if ( report_event( sn, "lapply",
-			"Error, changes made" ) != 0 ) {
+		if ( report_event( sn, event, "Error, changes made" ) != 0 ) {
 		    fprintf( stderr, "warning: could not report event\n" );
 		}
 	    }
 	} else {
 	    if ( network && report ) {
-		if ( report_event( sn, "lapply",
+		if ( report_event( sn, event,
 			"Error, no changes made" ) != 0 ) {
 		    fprintf( stderr, "warning: could not report event\n" );
 		}
