@@ -178,6 +178,7 @@ main( int argc, char **argv )
 {
     int			c, i, j, cmpval, err = 0, tcount = 0, candidate = 0;
     int			force = 0, ofd, fileloc = 0, match = 0;
+    int			merge_trans_only = 0;
     char		*file = NULL;
     char		npath[ 2 * MAXPATHLEN ];
     char		opath[ 2 * MAXPATHLEN ];
@@ -194,7 +195,7 @@ main( int argc, char **argv )
     FILE		*ofs;
     mode_t		mask;
 
-    while ( ( c = getopt( argc, argv, "D:fInu:Vv" ) ) != EOF ) {
+    while ( ( c = getopt( argc, argv, "D:fInTu:Vv" ) ) != EOF ) {
 	switch( c ) {
 	case 'D':
 	    radmind_path = optarg;
@@ -223,6 +224,9 @@ main( int argc, char **argv )
 	case 'v':
 	    verbose = 1;
 	    break;
+	case 'T':
+		merge_trans_only = 1;
+		break;
 	default:
 	    err++;
 	    break;
@@ -231,6 +235,9 @@ main( int argc, char **argv )
 
     tcount = argc - ( optind + 1 );	/* "+ 1" accounts for dest tran */
 
+	if ( merge_trans_only && force ) {
+		err++;
+	}
     if ( noupload && ( tcount > 2 ) ) {
 	err++;
     }
@@ -246,13 +253,13 @@ main( int argc, char **argv )
     }
 
     if ( err ) {
-	fprintf( stderr, "Usage: %s [-vIV] [ -D path ] [ -u umask ] ",
+	fprintf( stderr, "Usage: %s [-vIVT] [ -D path ] [ -u umask ] ",
 	    argv[ 0 ] );
 	fprintf( stderr, "transcript... dest\n" );
 	fprintf( stderr, "       %s -f [-vIV] [ -D path ] [ -u umask ] ",
 	    argv[ 0 ] );
 	fprintf( stderr, "transcript1 transcript2\n" );
-	fprintf( stderr, "       %s -n [-vIV] [ -D path ] [ -u umask ] ",
+	fprintf( stderr, "       %s -n [-vIVT] [ -D path ] [ -u umask ] ",
 	    argv[ 0 ] );
 	fprintf( stderr, "transcript1 transcript2 dest\n" );
 	exit( 2 );
@@ -347,7 +354,7 @@ main( int argc, char **argv )
 	    strcpy( cwd, argv[ argc - 1 ] );
 	} else {
 	    if ( snprintf( temp, MAXPATHLEN, "%s/%s", cwd, argv[ argc - 1 ] )
-		    > MAXPATHLEN - 1 ) {
+		    >= MAXPATHLEN ) {
 		fprintf( stderr, "%s/%s: path too long\n", cwd,
 		    argv[ argc - 1 ] );
 		exit( 2 );
@@ -360,20 +367,23 @@ main( int argc, char **argv )
 
 	/* Create file/tname dir */
 	if ( snprintf( npath, MAXPATHLEN, "%s/%s.%d", file_root, tran_name,
-		(int)getpid()) > MAXPATHLEN -1 ) {
+		(int)getpid()) >= MAXPATHLEN ) {
 	    fprintf( stderr, "%s/%s.%d: path too long\n", file_root, tran_name,
 		(int)getpid());
 	    exit( 2 );
 	}
-	if ( mkdir( npath, (mode_t)0777 ) != 0 ) {
-	    perror( npath );
-	    exit( 2 );
+	/* don't bother creating file/tname if only merging trans */
+	if ( !merge_trans_only ) {
+	    if ( mkdir( npath, (mode_t)0777 ) != 0 ) {
+		perror( npath );
+		exit( 2 );
+	    }
 	}
     }
 
     /* Create temp transcript/tname file */
     if ( snprintf( opath, MAXPATHLEN, "%s/%s.%d", tran_root, tran_name,
-	    (int)getpid()) > MAXPATHLEN - 1 ) {
+	    (int)getpid()) >= MAXPATHLEN ) {
 	fprintf( stderr, "%s/%s.%d: path too long\n", tran_root, tran_name,
 	    (int)getpid());
 	exit( 2 );
@@ -435,7 +445,7 @@ main( int argc, char **argv )
 			if ( snprintf( opath, MAXPATHLEN, "%s/%s/%s",
 				trans[ j ]->t_file_root,
 				trans[ j ]->t_tran_name,
-				trans[ j ]->t_filepath ) > MAXPATHLEN -1 ) {
+				trans[ j ]->t_filepath ) >= MAXPATHLEN ) {
 			    fprintf( stderr,
 				"%s/%s/%s: path too long\n",
 				trans[ j ]->t_file_root,
@@ -473,9 +483,12 @@ main( int argc, char **argv )
 		}
 		goto skipline;
 	    }
-	    /* output non-files */
-	    if ( *trans[ candidate ]->t_argv[ 0 ] != 'f'
-		    && *trans[ candidate ]->t_argv[ 0 ] != 'a' ) {
+	    /* output non-files, or if we're only merging transcripts 
+	     * and there is no file linking necessary
+	     */
+	    if (( *trans[ candidate ]->t_argv[ 0 ] != 'f'
+		    && *trans[ candidate ]->t_argv[ 0 ] != 'a')
+		    || merge_trans_only ) {
 		goto outputline;
 	    }
 
@@ -489,7 +502,7 @@ main( int argc, char **argv )
 		if ( snprintf( opath, MAXPATHLEN, "%s/%s%s",
 			trans[ candidate ]->t_file_root,
 			trans[ fileloc ]->t_tran_name,
-			trans[ candidate ]->t_filepath ) > MAXPATHLEN - 1 ) {
+			trans[ candidate ]->t_filepath ) >= MAXPATHLEN ) {
 		    fprintf( stderr, "%s/%s/%s: path too long\n",
 			trans[ candidate ]->t_file_root,
 			trans[ fileloc ]->t_tran_name,
@@ -500,7 +513,7 @@ main( int argc, char **argv )
 		if ( snprintf( opath, MAXPATHLEN, "%s/%s/%s",
 			trans[ candidate ]->t_file_root,
 			trans[ fileloc ]->t_tran_name,
-			trans[ candidate ]->t_filepath ) > MAXPATHLEN - 1 ) {
+			trans[ candidate ]->t_filepath ) >= MAXPATHLEN ) {
 		    fprintf( stderr, "%s/%s/%s: path too long\n",
 			trans[ candidate ]->t_file_root,
 			trans[ fileloc ]->t_tran_name,
@@ -515,7 +528,7 @@ main( int argc, char **argv )
 		    if ( snprintf( npath, MAXPATHLEN, "%s/%s.%d/%s",
 			    file_root, tran_name, (int)getpid(),
 			    trans[ candidate ]->t_filepath )
-			    > MAXPATHLEN - 1 ) {
+			    >= MAXPATHLEN ) {
 			fprintf( stderr, "%s/%s.%d/%s: path too long\n",
 			    file_root, tran_name, (int)getpid(),
 			    trans[ candidate ]->t_filepath );
@@ -524,7 +537,7 @@ main( int argc, char **argv )
 		} else {
 		    if ( snprintf( npath, MAXPATHLEN, "%s/%s.%d/%s",
 			    file_root, tran_name, (int)getpid(),
-			    trans[ candidate ]->t_filepath ) > MAXPATHLEN - 1 ) {
+			    trans[ candidate ]->t_filepath ) >= MAXPATHLEN ) {
 			fprintf( stderr, "%s/%s.%d/%s: path too long\n",
 			    file_root, tran_name, (int)getpid(),
 			    trans[ candidate ]->t_filepath );
@@ -534,7 +547,7 @@ main( int argc, char **argv )
 	    } else {
 		if ( snprintf( npath, MAXPATHLEN, "%s/%s/%s", file_root,
 			tran_name, trans[ candidate ]->t_filepath )
-			> MAXPATHLEN - 1 ) {
+			>= MAXPATHLEN ) {
 		    fprintf( stderr, "%s/%s/%s: path too long\n", 
 			file_root, tran_name, trans[ candidate ]->t_filepath );
 		    exit( 2 );
@@ -554,7 +567,7 @@ main( int argc, char **argv )
 				    "%s/%s.%d%s",
 				    file_root, tran_name, (int)getpid(), 
 				    trans[ candidate ]->t_filepath )
-				    > MAXPATHLEN - 1 ) {
+				    >= MAXPATHLEN ) {
 				fprintf( stderr,
 				    "%s/%s.%d%s: path too long\n",
 				    file_root, tran_name, (int)getpid(),
@@ -566,7 +579,7 @@ main( int argc, char **argv )
 				    "%s/%s.%d/%s",
 				    file_root, tran_name, (int)getpid(), 
 				    trans[ candidate ]->t_filepath )
-				    > MAXPATHLEN - 1 ) {
+				    >= MAXPATHLEN ) {
 				fprintf( stderr,
 				    "%s/%s.%d/%s: path too long\n",
 				    file_root, tran_name, (int)getpid(),
@@ -579,7 +592,7 @@ main( int argc, char **argv )
 			    if ( snprintf( npath, MAXPATHLEN,
 				    "%s/%s%s", file_root, tran_name,
 				    trans[ candidate ]->t_filepath )
-				    > MAXPATHLEN - 1 ) {
+				    >= MAXPATHLEN ) {
 				fprintf( stderr,
 				    "%s/%s%s: path too long\n", file_root,
 				    tran_name, trans[ candidate ]->t_filepath );
@@ -589,7 +602,7 @@ main( int argc, char **argv )
 			    if ( snprintf( npath, MAXPATHLEN,
 				    "%s/%s/%s", file_root, tran_name,
 				    trans[ candidate ]->t_filepath )
-				    > MAXPATHLEN - 1 ) {
+				    >= MAXPATHLEN ) {
 				fprintf( stderr,
 				    "%s/%s/%s: path too long\n", file_root,
 				    tran_name, trans[ candidate ]->t_filepath );
@@ -670,29 +683,34 @@ skipline:
     /* Rename temp transcript and file structure */
     if ( !force ) {
 	if ( snprintf( opath, MAXPATHLEN, "%s/%s.%d", file_root,
-		tran_name, (int)getpid()) > MAXPATHLEN - 1 ) {
+		tran_name, (int)getpid()) >= MAXPATHLEN ) {
 	    fprintf( stderr, "%s/%s.%d: path too long\n",
 		file_root, tran_name, (int)getpid());
 	    exit( 2 );
 	}
 	if ( snprintf( npath, MAXPATHLEN, "%s/%s", file_root, tran_name )
-		> MAXPATHLEN - 1 ) {
+		>= MAXPATHLEN ) {
 	    fprintf( stderr, "%s/%s: path too long\n", file_root, tran_name );
 	    exit( 2 );
 	}
-	if ( rename( opath, npath ) != 0 ) {
-	    perror( npath );
-	    exit( 2 );
+	/* don't try and move file/tname if doing client only merge,
+	 * it was never created.
+	 */
+	if ( !merge_trans_only ) {
+	    if ( rename( opath, npath ) != 0 ) {
+		perror( npath );
+		exit( 2 );
+	    }
 	}
     }
     if ( snprintf( opath, MAXPATHLEN, "%s/%s.%d", tran_root, tran_name,
-	    (int)getpid()) > MAXPATHLEN - 1 ) {
+	    (int)getpid()) >= MAXPATHLEN ) {
 	fprintf( stderr, "%s/%s.%d: path too long\n", tran_root, tran_name,
 	    (int)getpid());
 	exit( 2 );
     }
     if ( snprintf( npath, MAXPATHLEN, "%s/%s", tran_root, tran_name )
-	    > MAXPATHLEN - 1 ) {
+	    >= MAXPATHLEN ) {
 	fprintf( stderr, "%s/%s: path too long\n", tran_root, tran_name );
 	exit( 2 );
     }
