@@ -27,10 +27,12 @@ KFILE="_RADMIND_COMMANDFILE"
 SERVER="_RADMIND_HOST"
 TLSLEVEL="_RADMIND_AUTHLEVEL"
 EDITOR=${EDITOR:-vi}
+PAGER=${PAGER:-cat}
 USER=${SUDO_USER:-$USER}
 TMPDIR="${TMPDIR:=/tmp}"
 DEFAULTS="/etc/defaults/radmind"
 FSDIFFROOT="."
+DEFAULTWORKDIR="/"
 FLAG="_RADMIND_DIR/client/.RadmindRunning"
 CHECKEDOUT="_RADMIND_DIR/client/.CheckedOut"
 MAILDOMAIN="_RADMIND_MAIL_DOMAIN"
@@ -85,7 +87,7 @@ checkedout() {
 }
 
 usage() {
-    $ECHO "Usage:	$0 [ -ctV ] [ -h server ] [ -w authlevel ] { trip | update | create | auto | force | checkout | checkin } [ /path/or/file ]" >&2
+    $ECHO "Usage:	$0 [ -ctV ] [ -D working-directory ] [ -h server ] [ -w authlevel ] { trip | update | create | auto | force | checkout | checkin } [ /path/or/file ]" >&2
     exit 1
 }
 
@@ -187,7 +189,7 @@ update() {
 	exit 0
     fi
     if [ x"$opt" = x"interactive" ]; then
-	cat ${FTMP}
+	${PAGER} ${FTMP}
 	infocmp >/dev/null 2>&1
 	if [ $? -eq 0 ]; then
 	    can_edit=yes
@@ -278,7 +280,7 @@ if [ -f "${DEFAULTS}" ]; then
     . "${DEFAULTS}"
 fi
 
-while getopts %ch:Ilqr:tU:Vw: opt; do
+while getopts %cD:h:Ilqr:tU:Vw: opt; do
     case $opt in
     %)  PROGRESS="-%"
 	FPROGRESS="-%"
@@ -290,6 +292,9 @@ while getopts %ch:Ilqr:tU:Vw: opt; do
     c)	CHECKSUM="-csha1"
 	;;
 
+    D)  WORKDIR="$OPTARG"
+	;;
+
     h)	SERVER="$OPTARG"
     	;;
 
@@ -328,7 +333,7 @@ elif [ $# -ne 1 ]; then
     usage
 fi
 
-cd /
+cd "${WORKDIR:-$DEFAULTWORKDIR}"
 
 if [ ! -d "${RASHTMP}" ]; then
     mkdir -m 700 "${RASHTMP}"
@@ -388,7 +393,7 @@ update|up)
 
 create)
     # Since create does not modify the system, no need for checkedout
-    ktcheck -w ${TLSLEVEL} -h ${SERVER} -n -c sha1
+    ktcheck ${CASE} -w ${TLSLEVEL} -h ${SERVER} -n -c sha1
     case "$?" in
     0)	;;
 
