@@ -28,8 +28,8 @@
 #include "list.h"
 #include "wildcard.h"
 
+char * convert_path_type( char *path );
 int read_kfile( char *kfile, int location );
-static void t_new( int type, char *fullname, char *shortname, char *kfile );
 static void t_remove( int type, char *shortname );
 static void t_display( void );
 
@@ -37,6 +37,7 @@ struct transcript		*tran_head = NULL;
 static struct transcript	*prev_tran = NULL;
 extern int			edit_path;
 extern int			case_sensitive;
+extern int			tran_format;
 static char			*kdir;
 static struct list		*kfile_list;
 struct list			*special_list;
@@ -50,6 +51,55 @@ int				fs_minus;
 int				exclude_warnings = 0;
 FILE				*outtran;
 
+   char * 
+convert_path_type( char *path )
+{
+    int			len = 0;
+    static char		buf[ MAXPATHLEN ]; 
+
+    len = strlen( path );
+
+    if ( len == 1 ) {
+        if (( tran_format == T_ABSOLUTE ) && ( path[ 0 ] == '.' )) {
+            buf[ 0 ] = '/';
+            buf[ 1 ] = '\0';
+        } else if (( tran_format == T_RELATIVE ) && ( path[ 0 ] == '/' )) {
+            buf[ 0 ] = '.';
+            buf[ 1 ] = '\0';
+        } else {
+	    /* Nothing to convert */
+	    return( path );
+	}
+    } else {
+        if (( tran_format == T_ABSOLUTE ) && ( path[ 0 ] == '.' )) {
+            if ( path[ 1 ] == '/' ) {
+                /* Move past leading '.' */
+		path++;
+                if ( snprintf( buf, sizeof( buf ), "%s",
+			path ) >= MAXPATHLEN ) {
+		    return( NULL );
+                }
+            } else {
+                /* Instert leading '/' */
+		    if ( snprintf( buf, sizeof( buf ), "/%s",
+			    path ) >= MAXPATHLEN ) {
+		    return( NULL );
+                }
+            }
+        } else if (( tran_format == T_RELATIVE ) && ( path[ 0 ] == '/' )) {
+            /* Instert leading '.' */
+            if ( snprintf( buf, sizeof( buf ), ".%s", path ) >= MAXPATHLEN ) {
+		return( NULL );
+            }
+        } else { 
+	    /* Nothing to convert */
+	    return( path );
+	}
+    }
+
+    return( buf );
+}
+
     void 
 transcript_parse( struct transcript *tran ) 
 {
@@ -106,6 +156,14 @@ transcript_parse( struct transcript *tran )
 	    tran->t_fullname, tran->t_linenum );
 	exit( 2 );
     }
+
+    /* Convert path to match transcript type */
+    if (( epath = convert_path_type( epath )) == NULL ) {;
+	fprintf( stderr, "%s: line %d: path too long\n", tran->t_fullname,
+	    tran->t_linenum );
+	exit( 2 );
+    }
+
     if ( pathcasecmp( epath, tran->t_pinfo.pi_name, case_sensitive ) < 0 ) {
 	fprintf( stderr, "%s: line %d: bad sort order\n",
 	    tran->t_fullname, tran->t_linenum );
@@ -163,6 +221,27 @@ transcript_parse( struct transcript *tran )
 	break;
 
     case 'l':				    /* link */
+	if ( ac == 3 ) {	/* link without owner, group, mode */
+	    tran->t_pinfo.pi_stat.st_mode = 0777;
+	    tran->t_pinfo.pi_stat.st_uid = 0;
+	    tran->t_pinfo.pi_stat.st_gid = 0;
+	} else if ( ac == 6 ) { /* link with owner, group, mode */
+	    tran->t_pinfo.pi_stat.st_mode = strtol( argv[ 2 ], NULL, 8 );
+	    tran->t_pinfo.pi_stat.st_uid = atoi( argv[ 3 ] );
+	    tran->t_pinfo.pi_stat.st_gid = atoi( argv[ 4 ] );
+	} else {
+	    fprintf( stderr, "%s: line %d: expected 3 or 6 arguments, got %d\n",
+		    tran->t_fullname, tran->t_linenum, ac );
+	    exit( 2 );
+	}
+	if (( epath = decode( argv[ ac - 1 ] )) == NULL ) {
+	    fprintf( stderr, "%s: line %d: target path too long\n",
+		tran->t_fullname, tran->t_linenum );
+	    exit( 2 );
+	}
+	strcpy( tran->t_pinfo.pi_link, epath );
+	break;
+
     case 'h':				    /* hard */
 	if ( ac != 3 ) {
 	    fprintf( stderr, "%s: line %d: expected 3 arguments, got %d\n",
@@ -299,6 +378,16 @@ t_print( struct pathinfo *fs, struct transcript *tran, int flag )
 	break;
 
     case 'l':
+	fprintf( outtran, "%c %-37s\t%.4lo %5d %5d ", cur->pi_type, epath,
+		    (unsigned long)( T_MODE & cur->pi_stat.st_mode ),
+		    (int)cur->pi_stat.st_uid, (int)cur->pi_stat.st_gid );
+	if (( epath = encode( cur->pi_link )) == NULL ) {
+	    fprintf( stderr, "Filename too long: %s\n", cur->pi_link );
+	    exit( 2 );
+	}
+	fprintf( outtran, "%s\n", epath );
+	break;
+
     case 'h':
 	fprintf( outtran, "%c %-37s\t", cur->pi_type, epath );
 	if (( epath = encode( cur->pi_link )) == NULL ) {
@@ -504,7 +593,15 @@ t_compare( struct pathinfo *fs, struct transcript *tran )
 
     case 'l':			    /* link */
 	if ( tran->t_type != T_NEGATIVE ) {
-	    if ( strcmp( fs->pi_link, tran->t_pinfo.pi_link ) != 0 ) {
+	    if (( strcmp( fs->pi_link, tran->t_pinfo.pi_link ) != 0 )
+#ifdef HAVE_LCHOWN
+		 || ( fs->pi_stat.st_uid != tran->t_pinfo.pi_stat.st_uid ) ||
+		    ( fs->pi_stat.st_gid != tran->t_pinfo.pi_stat.st_gid )
+#endif /* HAVE_LCHOWN */
+#ifdef HAVE_LCHMOD
+		 || ( mode != tran_mode )
+#endif /* HAVE_LCHMOD */
+	    /* strcmp */ ) {
 		t_print( fs, tran, PR_STATUS );
 	    }
 	}
@@ -617,6 +714,17 @@ transcript_select( void )
 		continue;
 	    }
 
+	    /* If we match an exclude pattern, pretend we don't see it */
+	    if ( begin_tran->t_type != T_SPECIAL &&
+		    t_exclude( begin_tran->t_pinfo.pi_name )) {
+		if ( exclude_warnings ) {
+		    fprintf( stderr, "Warning: excluding %s\n",
+				begin_tran->t_pinfo.pi_name );
+		}
+		transcript_parse( begin_tran );
+		continue;
+	    }
+
 	    /* Don't look outside of the initial path. */
 	
 	    if ( !ischildcase( begin_tran->t_pinfo.pi_name, path_prefix,
@@ -720,7 +828,7 @@ transcript( char *path, struct stat *st, char *type,
     }
 }
 
-    static void
+    void
 t_new( int type, char *fullname, char *shortname, char *kfile ) 
 {
     struct transcript	 *new;
@@ -898,7 +1006,7 @@ read_kfile( char *kfile, int location )
     char	line[ MAXPATHLEN ];
     char	fullpath[ MAXPATHLEN ];
     char	*subpath;
-    char	*d_pattern;
+    char	*d_pattern, *path;
     char	**av;
     FILE	*fp;
 
@@ -1013,8 +1121,16 @@ read_kfile( char *kfile, int location )
 	case 'x':				/* exclude */
 	    if (( d_pattern = decode( av[ 1 ] )) == NULL ) {
 		fprintf( stderr, "%s: line %d: decode buffer too small\n",
-			kfile, linenum );
+		    kfile, linenum );
 	    }
+
+	    /* Convert path to match transcript type */
+	    if (( d_pattern = convert_path_type( d_pattern )) == NULL ) {
+		fprintf( stderr, "%s: line %d: path too long\n",
+		    kfile, linenum );
+		exit( 2 );
+	    }
+
 	    if ( minus ) {
 		list_remove( exclude_list, d_pattern );
 	    } else {
@@ -1028,13 +1144,22 @@ read_kfile( char *kfile, int location )
 	    break;
 
 	case 's':				/* special */
+	    path = av[ 1 ];
+
+	    /* Convert path to match transcript type */
+	    if (( path = convert_path_type( path )) == NULL ) {
+		fprintf( stderr, "%s: line %d: path too long\n",
+		    kfile, linenum );
+		exit( 2 );
+	    }
+
 	    if ( minus ) {
-		if ( list_check( special_list, av[ 1 ] )) {
-		    list_remove( special_list, av[ 1 ] );
+		if ( list_check( special_list, path )) {
+		    list_remove( special_list, path );
 		}
 	    } else {
-		if ( !list_check( special_list, av[ 1 ] )) {
-		    if ( list_insert( special_list, av[ 1 ] ) != 0 ) {
+		if ( !list_check( special_list, path )) {
+		    if ( list_insert( special_list, path ) != 0 ) {
 			perror( "list_insert" );
 			return( -1 );
 		    }
