@@ -10,6 +10,9 @@
 #define T_NEGATIVE	2 
 #define T_SPECIAL	3
 
+#define T_RELATIVE	0
+#define T_ABSOLUTE	1
+
 #define T_MOVE_TRAN	1
 #define T_MOVE_FS	2
 #define T_MOVE_BOTH	3 
@@ -65,6 +68,7 @@ void			transcript_init( char *kfile, int location );
 struct transcript	*transcript_select( void );
 void			transcript_parse( struct transcript * );
 void			transcript_free( void );
+void			t_new( int, char *, char *, char * );
 int			t_exclude( char *path );
 void			t_print( struct pathinfo *, struct transcript *, int );
 char			*hardlink( struct pathinfo * );
